@@ -82,7 +82,7 @@ def clip(max_delta) -> GradientTransformation:
 
   def update_fn(updates, state, params=None):
     del params
-    updates = jax.tree_multimap(
+    updates = jax.tree_map(
         lambda g: jnp.clip(g, -max_delta, max_delta), updates)
     return updates, state
 
@@ -120,9 +120,9 @@ def clip_by_global_norm(max_norm) -> GradientTransformation:
     # TODO(b/163995078): revert back to the following (faster) implementation
     # once analysed how it affects backprop through update (e.g. meta-gradients)
     # g_norm = jnp.maximum(max_norm, g_norm)
-    # updates = jax.tree_multimap(lambda t: (t / g_norm) * max_norm, updates)
+    # updates = jax.tree_map(lambda t: (t / g_norm) * max_norm, updates)
     trigger = g_norm < max_norm
-    updates = jax.tree_multimap(
+    updates = jax.tree_map(
         lambda t: jnp.where(trigger, t, (t / g_norm) * max_norm), updates)
     return updates, state
 
@@ -146,7 +146,7 @@ def trace(decay: float, nesterov: bool) -> GradientTransformation:
   """
 
   def init_fn(params):
-    return TraceState(trace=jax.tree_multimap(jnp.zeros_like, params))
+    return TraceState(trace=jax.tree_map(jnp.zeros_like, params))
 
   def update_fn(updates, state, params=None):
     del params
@@ -160,6 +160,44 @@ def trace(decay: float, nesterov: bool) -> GradientTransformation:
   return GradientTransformation(init_fn, update_fn)
 
 
+class ScaleByRssState(OptState):
+  """State holding the sum of gradient squares to date."""
+  sum_of_squares: Updates
+
+
+def scale_by_rss(initial_accumulator_value: float = 0.1, eps: float = 1e-7):
+  """Rescale updates by the root of the sum of all squared gradients to date.
+
+  References:
+    [Duchi et al, 2011](https://jmlr.org/papers/volume12/duchi11a/duchi11a.pdf)
+    [McMahan et al., 2010](https://arxiv.org/abs/1002.4908)
+
+  Args:
+    initial_accumulator_value: Starting value for accumulators, must be >= 0.
+    eps: A small floating point value to avoid zero denominator.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(params):
+    sum_of_squares = jax.tree_map(
+        lambda t: jnp.full_like(t, initial_accumulator_value), params)
+    return ScaleByRssState(sum_of_squares=sum_of_squares)
+
+  def update_fn(updates, state, params=None):
+    del params
+    sum_of_squares = jax.tree_multimap(
+        lambda g, t: jnp.square(g) + t, updates, state.sum_of_squares)
+    inv_sqrt_g_square = jax.tree_map(
+        lambda t: jnp.where(t > 0, jax.lax.rsqrt(t + eps), 0.0), sum_of_squares)
+    updates = jax.tree_multimap(
+        lambda scale, g: scale * g, inv_sqrt_g_square, updates)
+    return updates, ScaleByRssState(sum_of_squares=sum_of_squares)
+
+  return GradientTransformation(init_fn, update_fn)
+
+
 class ScaleByRmsState(OptState):
   """State for exponential root mean-squared (RMS)-normalized updates."""
   nu: Updates
@@ -185,14 +223,14 @@ def scale_by_rms(decay: float = 0.9, eps: float = 1e-8):
   """
 
   def init_fn(params):
-    nu = jax.tree_multimap(jnp.zeros_like, params)  # second moment
+    nu = jax.tree_map(jnp.zeros_like, params)  # second moment
     return ScaleByRmsState(nu=nu)
 
   def update_fn(updates, state, params=None):
     del params
     nu = _update_moment(updates, state.nu, decay, 2)
     updates = jax.tree_multimap(
-        lambda g, n: g / (jnp.sqrt(n + eps)), updates, nu)
+        lambda g, n: g * jax.lax.rsqrt(n + eps), updates, nu)
     return updates, ScaleByRmsState(nu=nu)
 
   return GradientTransformation(init_fn, update_fn)
@@ -220,8 +258,8 @@ def scale_by_stddev(
   """
 
   def init_fn(params):
-    mu = jax.tree_multimap(jnp.zeros_like, params)  # First moment
-    nu = jax.tree_multimap(jnp.zeros_like, params)  # Second moment
+    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
+    nu = jax.tree_map(jnp.zeros_like, params)  # Second moment
     return ScaleByRStdDevState(mu=mu, nu=nu)
 
   def update_fn(updates, state, params=None):
@@ -229,7 +267,8 @@ def scale_by_stddev(
     mu = _update_moment(updates, state.mu, decay, 1)
     nu = _update_moment(updates, state.nu, decay, 2)
     updates = jax.tree_multimap(
-        lambda g, m, n: g / jnp.sqrt(n - jnp.square(m) + eps), updates, mu, nu)
+        lambda g, m, n: g * jax.lax.rsqrt(n - jnp.square(m) + eps), updates, mu,
+        nu)
     return updates, ScaleByRStdDevState(mu=mu, nu=nu)
 
   return GradientTransformation(init_fn, update_fn)
@@ -256,12 +295,20 @@ def _safe_int32_increment(count):
   """
   chex.assert_type(count, jnp.int32)
   max_int32_value = jnp.iinfo(jnp.int32).max
-  return jnp.where(count < max_int32_value, count + 1, max_int32_value)
+  one = jnp.array(1, dtype=jnp.int32)
+  return jnp.where(count < max_int32_value, count + one, max_int32_value)
 
 
-def scale_by_adam(
-    b1: float = 0.9, b2: float = 0.999,
-    eps: float = 1e-8, eps_root: float = 0.0) -> GradientTransformation:
+def _bias_correction(moment, decay, count):
+  """Perform bias correction. This becomes a no-op as count goes to infinity."""
+  bias_correction = 1 - decay**count
+  return jax.tree_map(lambda t: t / bias_correction.astype(t.dtype), moment)
+
+
+def scale_by_adam(b1: float = 0.9,
+                  b2: float = 0.999,
+                  eps: float = 1e-8,
+                  eps_root: float = 0.0) -> GradientTransformation:
   """Rescale updates according to the Adam algorithm.
 
   References:
@@ -279,20 +326,20 @@ def scale_by_adam(
   """
 
   def init_fn(params):
-    mu = jax.tree_multimap(jnp.zeros_like, params)  # First moment
-    nu = jax.tree_multimap(jnp.zeros_like, params)  # Second moment
+    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
+    nu = jax.tree_map(jnp.zeros_like, params)  # Second moment
     return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
 
   def update_fn(updates, state, params=None):
     del params
     mu = _update_moment(updates, state.mu, b1, 1)
     nu = _update_moment(updates, state.nu, b2, 2)
-    mu_hat = jax.tree_multimap(lambda t: t / (1 - b1 ** (state.count + 1)), mu)
-    nu_hat = jax.tree_multimap(lambda t: t / (1 - b2 ** (state.count + 1)), nu)
+    count_inc = _safe_int32_increment(state.count)
+    mu_hat = _bias_correction(mu, b1, count_inc)
+    nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
         lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
-    return updates, ScaleByAdamState(
-        count=_safe_int32_increment(state.count), mu=mu, nu=nu)
+    return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
 
   return GradientTransformation(init_fn, update_fn)
 
@@ -316,12 +363,58 @@ def scale(step_size: float) -> GradientTransformation:
 
   def update_fn(updates, state, params=None):
     del params
-    updates = jax.tree_multimap(lambda g: step_size * g, updates)
+    updates = jax.tree_map(lambda g: step_size * g, updates)
     return updates, state
 
   return GradientTransformation(init_fn, update_fn)
 
 
+class ScaleByBeliefState(OptState):
+  """State for the rescaling by AdaBelief algorithm."""
+  count: jnp.ndarray  # shape=(), dtype=jnp.int32.
+  mu: Updates
+  nu: Updates
+
+
+def scale_by_belief(
+    b1: float = 0.9, b2: float = 0.999,
+    eps: float = 0., eps_root: float = 1e-16) -> GradientTransformation:
+  """Rescale updates according to the AdaBelief algorithm.
+
+  References:
+    [Zhuang et al, 2020](https://arxiv.org/abs/2010.07468)
+
+  Args:
+    b1: decay rate for the exponentially weighted average of grads.
+    b2: decay rate for the exponentially weighted average of variance of grads.
+    eps: term added to the denominator to improve numerical stability.
+    eps_root: term added to the denominator inside the square-root to improve
+      numerical stability when backpropagating gradients through the rescaling.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(params):
+    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
+    s = jax.tree_map(jnp.zeros_like, params)  # Second Central moment
+    return ScaleByBeliefState(count=jnp.zeros([], jnp.int32), mu=mu, nu=s)
+
+  def update_fn(updates, state, params=None):
+    del params
+    mu = _update_moment(updates, state.mu, b1, 1)
+    prediction_error = jax.tree_multimap(lambda g, m: g-m, updates, state.mu)
+    nu = _update_moment(prediction_error, state.nu, b2, 2)
+    count_inc = _safe_int32_increment(state.count)
+    mu_hat = _bias_correction(mu, b1, count_inc)
+    nu_hat = _bias_correction(nu, b2, count_inc)
+    updates = jax.tree_multimap(
+        lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
+    return updates, ScaleByBeliefState(count=count_inc, mu=mu, nu=nu)
+
+  return GradientTransformation(init_fn, update_fn)
+
+
 class AdditiveWeightDecayState(NamedTuple):
   """The decay transformation is stateless."""
 
@@ -340,8 +433,8 @@ def additive_weight_decay(weight_decay: float = 0.0) -> GradientTransformation:
     return AdditiveWeightDecayState()
 
   def update_fn(updates, state, params):
-    updates = jax.tree_multimap(lambda g, p: g + weight_decay * p, updates,
-                                params)
+    updates = jax.tree_multimap(
+        lambda g, p: g + weight_decay * p, updates, params)
     return updates, state
 
   return GradientTransformation(init_fn, update_fn)
@@ -368,8 +461,9 @@ def scale_by_schedule(step_size_fn: Callable[[jnp.ndarray], jnp.ndarray]):
 
   def update_fn(updates, state, params=None):
     del params
-    updates = jax.tree_multimap(lambda g: step_size_fn(state.count) * g,
-                                updates)
+    step_size = step_size_fn(state.count)
+    updates = jax.tree_map(
+        lambda g: jnp.array(step_size, dtype=g.dtype) * g, updates)
     return updates, ScaleByScheduleState(
         count=_safe_int32_increment(state.count))
 
@@ -440,8 +534,10 @@ def scale_by_fromage(
       scaled_param_correction = param * (mult - 1.)
       return scaled_update + scaled_param_correction
 
+    count_inc = _safe_int32_increment(state.count)
     updates = jax.tree_multimap(_scale_update, updates, params)
-    return updates, ScaleByFromageState(count=state.count + 1)
+
+    return updates, ScaleByFromageState(count=count_inc)
 
   return GradientTransformation(init_fn, update_fn)
 
@@ -514,15 +610,16 @@ def add_noise(eta: float, gamma: float, seed: int) -> GradientTransformation:
     del params
     num_vars = len(jax.tree_leaves(updates))
     treedef = jax.tree_structure(updates)
-    variance = eta / (1 + state.count) ** gamma
+    count_inc = _safe_int32_increment(state.count)
+    variance = eta / count_inc**gamma
     all_keys = jax.random.split(state.rng_key, num=num_vars + 1)
     noise = jax.tree_multimap(
-        lambda g, k: jax.random.normal(k, shape=g.shape),
+        lambda g, k: jax.random.normal(k, shape=g.shape, dtype=g.dtype),
         updates, jax.tree_unflatten(treedef, all_keys[1:]))
     updates = jax.tree_multimap(
-        lambda g, n: g + variance * n, updates, noise)
-    return updates, AddNoiseState(
-        count=_safe_int32_increment(state.count), rng_key=all_keys[0])
+        lambda g, n: g + variance.astype(g.dtype) * n,
+        updates, noise)
+    return updates, AddNoiseState(count=count_inc, rng_key=all_keys[0])
 
   return GradientTransformation(init_fn, update_fn)
 
@@ -544,7 +641,7 @@ def apply_every(k: int = 1) -> GradientTransformation:
   """
 
   def init_fn(params):
-    grad_acc = jax.tree_multimap(jnp.zeros_like, params)
+    grad_acc = jax.tree_map(jnp.zeros_like, params)
     return ApplyEvery(count=jnp.zeros([], jnp.int32), grad_acc=grad_acc)
 
   def update_fn(updates, state, params=None):
@@ -554,7 +651,40 @@ def apply_every(k: int = 1) -> GradientTransformation:
     grad_acc = jax.tree_multimap(
         lambda g, ga: acc * ga + g, updates, state.grad_acc)
     emit = c == (k - 1)
-    updates = jax.tree_multimap(lambda ga: emit * ga, grad_acc)
-    return updates, ApplyEvery(count=(state.count + 1) % k, grad_acc=grad_acc)
+    updates = jax.tree_map(lambda ga: emit * ga, grad_acc)
+    count_inc = _safe_int32_increment(state.count)
+    return updates, ApplyEvery(count=count_inc % k, grad_acc=grad_acc)
+
+  return GradientTransformation(init_fn, update_fn)
+
+
+def _subtract_mean(g):
+  if len(g.shape) > 1:
+    return g - g.mean(tuple(range(1, len(g.shape))), keepdims=True)
+  else:
+    return g
+
+
+class CentralState(OptState):
+  """The `centralize` transformation is stateless."""
+
+
+def centralize() -> GradientTransformation:
+  """Centralize gradients.
+
+  References:
+    [Yong et al, 2020](https://arxiv.org/abs/2004.01461)
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(_):
+    return CentralState()
+
+  def update_fn(updates, state, params=None):
+    del params
+    updates = jax.tree_map(_subtract_mean, updates)
+    return updates, state
 
   return GradientTransformation(init_fn, update_fn)
